
#include	"dac_ctrl.h"
#include	"CtrlDac3224x192.h"

#include	<process.h> 

BRDctrl_StreamCBufAlloc g_buf_dscr;

static void DisplayError(S32 status, TCHAR* cmd_str)
{
	S32	real_status = BRD_errext(status);
	TCHAR msg[255];
	switch(real_status)
	{
	case BRDerr_OK:
		sprintf_s(msg, "%s: BRDerr_OK\n", cmd_str);
		break;
	case BRDerr_BAD_MODE:
		sprintf_s(msg, "%s: BRDerr_BAD_MODE\n", cmd_str);
		break;
	case BRDerr_INSUFFICIENT_SERVICES:
		sprintf_s(msg, "%s: BRDerr_INSUFFICIENT_SERVICES\n", cmd_str);
		break;
	case BRDerr_BAD_PARAMETER:
		sprintf_s(msg, "%s: BRDerr_BAD_PARAMETER\n", cmd_str);
		break;
	case BRDerr_BUFFER_TOO_SMALL:
		sprintf_s(msg, "%s: BRDerr_BUFFER_TOO_SMALL\n", cmd_str);
		break;
	default:
		sprintf_s(msg, "%s: Unknown error, status = %d\n", cmd_str, real_status);
		break;
	}
	printf(msg);
}

//   
S32 DacSettings(BRD_Handle hDAC, int idx, char* srvName, char* iniFileName)
{
	S32		status;

	//ULONG master = BRDims_SINGLE; //    () 
	//status = BRD_ctrl(hDAC, 0, BRDctrl_DAC_SETMASTER, &master);

	BRD_DacCfg dac_cfg;
	status = BRD_ctrl(hDAC, 0, BRDctrl_DAC_GETCFG, &dac_cfg);
	printf("DAC Config: FIFO size = %d kBytes\n", dac_cfg.FifoSize / 1024);
	ULONG bBufSize = dac_cfg.FifoSize; //      

	//    
	TCHAR iniFilePath[MAX_PATH];
	TCHAR iniSectionName[MAX_PATH];
	GetCurrentDirectory(sizeof(iniFilePath)/sizeof(TCHAR), iniFilePath);
	lstrcat(iniFilePath, iniFileName);
	sprintf_s(iniSectionName, "device%d_%s%d", idx, srvName, 0);

	BRD_IniFile ini_file;
	lstrcpy(ini_file.fileName, iniFilePath);
	lstrcpy(ini_file.sectionName, iniSectionName);
	status = BRD_ctrl(hDAC, 0, BRDctrl_DAC_READINIFILE, &ini_file);

	//         
	BRD_SyncMode sync_mode;
	status = BRD_ctrl(hDAC, 0, BRDctrl_DAC_GETSYNCMODE, &sync_mode);
	if(strstr(srvName, "DAC1624X192") || strstr(srvName, "DAC818X800"))
	{
		if(BRD_errcmp(status, BRDerr_OK))
			printf("BRDctrl_DAC_GETSYNCMODE: source = %d, value = %.2f MHz, rate = %.2f kHz\n", 
											sync_mode.clkSrc, sync_mode.clkValue/1000000, sync_mode.rate/1000);
		else
			DisplayError(status, "BRDctrl_DAC_GETSYNCMODE");
	}
	else
	{
		if(BRD_errcmp(status, BRDerr_OK))
			printf("BRDctrl_DAC_GETSYNCMODE: source = %d, value = %.2f MHz, rate = %.2f MHz\n", 
											sync_mode.clkSrc, sync_mode.clkValue/1000000, sync_mode.rate/1000000);
		else
			DisplayError(status, "BRDctrl_DAC_GETSYNCMODE");
	}
	//    
	BRD_StartMode start;
	status = BRD_ctrl(hDAC, 0, BRDctrl_DAC_GETSTARTMODE, &start);
	if(BRD_errcmp(status, BRDerr_OK))
		printf("BRDctrl_DAC_GETSTARTMODE: start source = %d\n", start.startSrc);
	else
		DisplayError(status, "BRDctrl_DAC_GETSTARTMODE");

	//    
	ULONG chan_mask = 0;
	status = BRD_ctrl(hDAC, 0, BRDctrl_DAC_GETCHANMASK, &chan_mask);
	//status = BRD_ctrl(hDAC, 0, BRDctrl_DAC_SETCHANMASK, &chan_mask);
	if(BRD_errcmp(status, BRDerr_OK))
		printf("BRDctrl_DAC_GETCHANMASK: chan_mask = %0X\n", chan_mask);
	else
		DisplayError(status, "BRDctrl_DAC_GETCHANMASK");
	U32 i = 0;
	S32 numChan  = 0;
	//numChan = (chan_mask & 0x1) + ((chan_mask >> 1) & 0x1) + ((chan_mask >> 2) & 0x1) + ((chan_mask >> 3) & 0x1);
	for(i = 0; i < 32; i++)
		numChan += (chan_mask >> i) & 0x1;


	if(strstr(srvName, "DAC3224X192"))
	{
		ULONG FunctionalMode;
		if(sync_mode.rate < MAX_SINGLE_RATE)
			FunctionalMode = 0; // SINGLE
		else
			if(sync_mode.rate < MAX_DOUBLE_RATE)
				FunctionalMode = 1; // DOUBLE
			else
				FunctionalMode = 2; // QUAD

		BRD_DacRegWrite dac_reg;
		BRD_DacSpec spec;
		spec.command = DAC3224X192cmd_WRITE;
		spec.arg = &dac_reg;

		dac_reg.progMask = 0xf; //    
		dac_reg.data = 0x24 | FunctionalMode; // 0x24 - for save default value

		dac_reg.addr = 0x6; // MixingControl Pair 1
		status = BRD_ctrl(hDAC, 0, BRDctrl_DAC_SETSPECIFIC, &spec);
		if(BRD_errcmp(status, BRDerr_OK))
			printf("BRDctrl_DAC_SETSPECIFIC: MixingControl Pair 1\n");
		else
			DisplayError(status, "BRDctrl_DAC_SETSPECIFIC");

		dac_reg.addr = 0x9; // MixingControl Pair 2
		status = BRD_ctrl(hDAC, 0, BRDctrl_DAC_SETSPECIFIC, &spec);
		if(BRD_errcmp(status, BRDerr_OK))
			printf("BRDctrl_DAC_SETSPECIFIC: MixingControl Pair 2\n");
		else
			DisplayError(status, "BRDctrl_DAC_SETSPECIFIC");

		dac_reg.addr = 0xC; // MixingControl Pair 3
		status = BRD_ctrl(hDAC, 0, BRDctrl_DAC_SETSPECIFIC, &spec);
		if(BRD_errcmp(status, BRDerr_OK))
			printf("BRDctrl_DAC_SETSPECIFIC: MixingControl Pair 3\n");
		else
			DisplayError(status, "BRDctrl_DAC_SETSPECIFIC");

		dac_reg.addr = 0xF; // MixingControl Pair 4
		status = BRD_ctrl(hDAC, 0, BRDctrl_DAC_SETSPECIFIC, &spec);
		if(BRD_errcmp(status, BRDerr_OK))
			printf("BRDctrl_DAC_SETSPECIFIC: MixingControl Pair 4\n");
		else
			DisplayError(status, "BRDctrl_DAC_SETSPECIFIC");
	}

	return numChan;
}

#include	<conio.h>
//         FIFO
S32 OutputFifo(BRD_Handle hDAC, PVOID pSig, ULONG bBufSize, ULONG isCycling)
{
	printf("\nCycling output into DAC (FIFO)\n");

	S32		status;
	ULONG Status = 0;
	ULONG Enable = 0;

//	status = BRD_ctrl(hDAC, 0, BRDctrl_DAC_FIFOSTATUS, &Status);

	status = BRD_ctrl(hDAC, 0, BRDctrl_DAC_ENABLE, &Enable); //   
	if(BRD_errcmp(status, BRDerr_OK))
		printf("BRDctrl_DAC_ENABLE: Enable = %d\n", Enable);
	else
		DisplayError(status, "BRDctrl_DAC_ENABLE");

	status = BRD_ctrl(hDAC, 0, BRDctrl_DAC_FIFORESET, NULL); //  FIFO 
	if(BRD_errcmp(status, BRDerr_OK))
		printf("BRDctrl_DAC_FIFORESET: OK\n");
	else
		DisplayError(status, "BRDctrl_DAC_FIFORESET");

	//Enable = 1;
	//status = BRD_ctrl(hDAC, 0, BRDctrl_DAC_ENABLE, &Enable); //   
	//if(BRD_errcmp(status, BRDerr_OK))
	//	printf("BRDctrl_DAC_ENABLE: Enable = %d\n", Enable);
	//else
	//	DisplayError(status, "BRDctrl_DAC_ENABLE");

	BRD_DataBuf data_buf;
	data_buf.pData = pSig;
	data_buf.size = bBufSize;
	status = BRD_ctrl(hDAC, 0, BRDctrl_DAC_PUTDATA, &data_buf);
	if(BRD_errcmp(status, BRDerr_OK))
		printf("BRDctrl_DAC_PUTDATA: OK\n");
	else
		DisplayError(status, "BRDctrl_DAC_PUTDATA");

	ULONG cycling = isCycling; // 1 -  
	status = BRD_ctrl(hDAC, 0, BRDctrl_DAC_SETCYCLMODE, &cycling);

	Enable = 1;
	status = BRD_ctrl(hDAC, 0, BRDctrl_DAC_ENABLE, &Enable); //   
	if(BRD_errcmp(status, BRDerr_OK))
		printf("BRDctrl_DAC_ENABLE: Enable = %d\n", Enable);
	else
		DisplayError(status, "BRDctrl_DAC_ENABLE");

    if(isCycling)
	{
		printf("Hit any key to abort...\n");
		_getch();
	}
	else
	{
		status = BRD_ctrl(hDAC, 0, BRDctrl_DAC_FIFOSTATUS, &Status);
	}

	Enable = 0;
	status = BRD_ctrl(hDAC, 0, BRDctrl_DAC_ENABLE, &Enable); //   
	if(BRD_errcmp(status, BRDerr_OK))
		printf("BRDctrl_DAC_ENABLE: Enable = %d\n", Enable);
	else
		DisplayError(status, "BRDctrl_DAC_ENABLE");

	cycling = 0; //   
	status = BRD_ctrl(hDAC, 0, BRDctrl_DAC_SETCYCLMODE, &cycling);

	return status;
}

//       -   
S32 OutputUseDMA(BRD_Handle hDAC, int idx, ULONG bBufSize, ULONG isCycling)
{
//	printf("\nOne time output into DAC (DMA)\n");
	printf("\nCycling output into DAC (DMA)\n");
	S32		status;
	ULONG Status = 0;

	ULONG Enable = 0;
	status = BRD_ctrl(hDAC, 0, BRDctrl_DAC_ENABLE, &Enable); //   
	status = BRD_ctrl(hDAC, 0, BRDctrl_DAC_FIFORESET, NULL); //  FIFO 

	//     
	ULONG tetrad;
	status = BRD_ctrl(hDAC, 0, BRDctrl_DAC_GETSRCSTREAM, &tetrad); //     
	status = BRD_ctrl(hDAC, 0, BRDctrl_STREAM_SETSRC, &tetrad);

	BRDctrl_StreamCBufStart start_pars;
	start_pars.isCycle = isCycling; // 1 -  
	status = BRD_ctrl(hDAC, 0, BRDctrl_STREAM_CBUF_START, &start_pars); //  
	Enable = 1;
	status = BRD_ctrl(hDAC, 0, BRDctrl_DAC_ENABLE, &Enable); //   

	if(isCycling)
	{
		int fl_break = 0;
		printf("Hit any key to abort...\n");
		while(!fl_break & !_kbhit())
		{
			ULONG msTimeout = 20000; //       20 .
			status = BRD_ctrl(hDAC, 0, BRDctrl_STREAM_CBUF_WAITBLOCK, &msTimeout);
			if(BRD_errcmp(status, BRDerr_WAIT_TIMEOUT))
			{	//    -,  
				status = BRD_ctrl(hDAC, 0, BRDctrl_STREAM_CBUF_STOP, NULL);
				DisplayError(status, "TIME-OUT");
				fl_break = 1;
			}
		}
		if(!fl_break)
			_getch();
	}
	else
	{
		ULONG msTimeout = 20000; //      20 .
		status = BRD_ctrl(hDAC, 0, BRDctrl_STREAM_CBUF_WAITBUF, &msTimeout);
		if(BRD_errcmp(status, BRDerr_WAIT_TIMEOUT))
		{	//    -,  
			status = BRD_ctrl(hDAC, 0, BRDctrl_STREAM_CBUF_STOP, NULL);
			DisplayError(status, "TIME-OUT");
		}
	}

	Enable = 0;
	status = BRD_ctrl(hDAC, 0, BRDctrl_DAC_ENABLE, &Enable); //   

	return status;
}

//         
S32 AllocBuf(BRD_Handle hDAC, PVOID* pSig, ULONG bBufSize, ULONG bufType)
{
	S32		status;

	void* pBuffer = NULL;
	if(2 == bufType)
	{
		pBuffer = malloc(bBufSize);
//		pBuffer = VirtualAlloc(NULL, bBufSize, MEM_COMMIT, PAGE_READWRITE);
		if(!pBuffer)
		{
			printf("VirtualAlloc() is error!!!\n");
			return -1; // error
		}
	}
	g_buf_dscr.dir = BRDstrm_DIR_OUT;
	g_buf_dscr.isCont = bufType; // 0 -      , 1 -  
	g_buf_dscr.blkNum = 1;
	g_buf_dscr.blkSize = bBufSize;
	g_buf_dscr.ppBlk = new PVOID[g_buf_dscr.blkNum];
	if(g_buf_dscr.isCont == 2)
		g_buf_dscr.ppBlk[0] = pBuffer;
	status = BRD_ctrl(hDAC, 0, BRDctrl_STREAM_CBUF_ALLOC, &g_buf_dscr);
	if(!BRD_errcmp(status, BRDerr_OK) && !BRD_errcmp(status, BRDerr_PARAMETER_CHANGED))
		DisplayError(status, "BRDctrl_STREAM_CBUF_ALLOC");
	if(BRD_errcmp(status, BRDerr_PARAMETER_CHANGED))
	{
		printf("Warning!!! BRDctrl_STREAM_CBUF_ALLOC: BRDerr_PARAMETER_CHANGED\n");
		status = BRDerr_OK;
	}
	*pSig = g_buf_dscr.ppBlk[0];
	return status;
}

//         
S32 FreeBuf(BRD_Handle hDAC)
{
	S32		status;
	status = BRD_ctrl(hDAC, 0, BRDctrl_STREAM_CBUF_FREE, NULL);
	if(!BRD_errcmp(status, BRDerr_OK))
		DisplayError(status, "BRDctrl_STREAM_CBUF_FREE");
	if(g_buf_dscr.isCont == 2)
		free(g_buf_dscr.ppBlk[0]);
//		VirtualFree(g_buf_dscr.ppBlk[0], 0, MEM_RELEASE);
	delete[] g_buf_dscr.ppBlk;
	return status;
}
